-- PROJECT:  MATH_EVAL
-- VERSION:  1.1.0  3.12.2014   (created on LinuxMint 17, using Geany editor).
-- AUTHOR:   Shian Lee
-- LANGUAGE: Euphoria 4.1.0  <http://openeuphoria.org/>
-- LICENSE:  Free. Use at your own risk.
-- --------------------------------------------------------------------------
-- PURPOSE:  command-line-expression-calculator is a command-line utility
--           program for using math_eval.e.
--           * make sure to review the 'USER DEFINED CODE' block below,
--             before running this program.
-- NOTE:     * don't get confused between radians and degrees modes, for
--             example 'cos(10)' returns different result in each mode...
-- EXAMPLE:  * eui ./ecalc.ex -h                    -->  display help
--           * eui ./ecalc.ex "abs(-1)*9.3-#ff/9"   -->  calc expression
--              Result = -19.0333
--           * eui ./ecalc.ex -d "cos(a)+b"         -->  calc using degrees
--              Enter value or expression for 'a': 270+15
--              Enter value or expression for 'b': 2.5
--              Result = 2.75882
-- SEEALSO:  'exprcalc.ex' console mode demo program.
-- ==========================================================================



-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~ USER DEFINED CODE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

-- pathname for math_eval.e include file:
include ./math_eval.e

-- ~~~~~~~~~~~~~~~~~~~~~~~~ END OF USER DEFINED CODE ~~~~~~~~~~~~~~~~~~~~~~~~



-- include Euphoria 4.1 files:
include std/types.e
include std/io.e
include std/console.e



-- PURPOSE: display help and abort().
-- SEEALSO: main
procedure display_help()
    puts(STDOUT,
        MATH_EVAL_VERSION & "\n\n" &
        "Usage: ecalc [OPTION] \"MATH-EXPRESSION\"\n" &
        "Calculate math expression using Euphoria 4.1 syntax.\n\n" &
        "MATH-EXPRESSION must be enclosed with \"\".\n\n" &
        "Options:\n" &
        "  -d       use degrees in trigonometry functions (not radians)\n" &
        "  -h, -?   display this help and exit\n\n" &
        "New version of math_eval: <http://rapideuphoria.com/archive.htm>\n" &
        "OpenEuphoria manual: <http://openeuphoria.org/docs/>\n" &
        "For complete documentation, view: math_eval.doc\n"
    )

    abort(1)
end procedure



-- PURPOSE: calculate expression using math_eval() function, and abort().
-- DETAILS: expr is a math expression to calculate.
--          if deg then use degrees else use radians in calculations.
-- SEEALSO: main
procedure calc_expr(string expr, boolean deg)
    object   result
    sequence v, id = get_identifiers(expr, GET_ID_VARIABLES)

    if id[1] = GET_ID_SUCCESS then
        -- sequence for storing user variables:
        v = {
            id[2],                      -- [1] variable names
            repeat("", length(id[2]))   -- [2] variable values or expressions
        }

        -- prompt for variables value/expression:
        for i = 1 to length(v[1]) do
            v[2][i] = prompt_string(" Enter value or expression for '" &
                                                            v[1][i] & "': ")
        end for

        -- calculate expression:
        result = math_eval(expr, deg, v)

    else -- id[1] = GET_ID_FAIL:
        result = id[2]  -- error message string
    end if

    -- print result atom or error message:
    if atom(result) then
        puts(STDOUT, sprintf(" Result = %g\n", result))
    else
        puts(STDERR, 32 & result & '\n')
    end if

    abort(sequence(result))
end procedure



-- PURPOSE: startup routine that runs the program.
constant
    CMD = command_line(),
    LEN_CMD = length(CMD)
procedure main()
    string  expr        -- command-line "math expression".
    boolean deg, help   -- deg: FALSE=radians/TRUE=degrees.

    if LEN_CMD > 2 then
        help = find(CMD[3], {"-h", "-?"}) > 0

        if not help then
            deg  = equal(CMD[3], "-d")

            if LEN_CMD = 3 + deg then
                expr = CMD[3 + deg]
                calc_expr(expr, deg)    -- calculate expression and abort()
            end if
        end if
    end if

    display_help()      -- show help and abort()
end procedure



main()



-- End of file.
