-- PROJECT:  MATH_EVAL
-- VERSION:  1.1.0  3.12.2014   (created on LinuxMint 17, using Geany editor).
-- AUTHOR:   Shian Lee
-- LANGUAGE: Euphoria 4.1.0  <http://openeuphoria.org/>
-- LICENSE:  Free. Use at your own risk.
-- --------------------------------------------------------------------------
-- PURPOSE:  expression-calculator is a console mode demo program for using
--           math_eval.e.
--           * make sure to review the 'USER DEFINED CODE' block below,
--             before running this program.
-- DETAILS:  * sequence 'data' stores all the expressions - see get_data().
--           * integer 'ix' is the current index in 'data' (expression index).
-- NOTE:     * don't get confused between radians and degrees modes, for
--             example 'cos(10)' returns different result in each mode...
-- EXAMPLE:  eui ./exprcalc.ex      -->  run the program.
-- SEEALSO:  'ecalc.ex' command-line utility program.
-- ==========================================================================



-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~ USER DEFINED CODE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

-- pathname for math_eval.e include file:
include ./math_eval.e

-- pathname for data file (for keeping the expressions):
constant DATA_FILE = "/home/shian/exprcalc.dat"

-- pathname for the Euphoria 4.1 interpreter (for comparing results):
constant EU_INTERPRETER = "eui"

-- pathname for temp file to be interpreted by Euphoria:
constant EU_TEMP_FILE = "/home/shian/~eui_tmp" & ".ex"

-- ~~~~~~~~~~~~~~~~~~~~~~~~ END OF USER DEFINED CODE ~~~~~~~~~~~~~~~~~~~~~~~~



-- include Euphoria 4.1 files:
include std/console.e
include std/text.e
include std/get.e
include std/types.e
include std/io.e
include std/graphics.e
include std/utils.e



-- UTILITIES ================================================================

-- PURPOSE: save startup video configuration.
constant VC = video_config(),
  SCRN_ROWS = VC[VC_SCRNLINES],
  SCRN_COLS = VC[VC_SCRNCOLS]



-- PURPOSE: set the screen colors (the default is always WHITE on BLACK).
procedure set_colors(integer text = WHITE, integer bk = BLACK)
    text_color(text)
    bk_color(bk)
end procedure



-- PURPOSE: clear screen and print menubar, using default colors.
--          this function also reset colors to default colors before return.
-- DETAILS: start_line is the top line for clear screen. 0=don't clear screen.
--          menubar is a string to print at position(1, 1). ""=don't print bar.
-- NOTE:    unlike clear_screen(), scroll() does not add lines to the console
--          buffer (at least in my Linux); i.e. we stay in the same screen.
-- SEEALSO: verify_screen_size
procedure clear_console(integer start_line = 1, string menubar = "")
    integer len = length(menubar)

    -- display menu bar:
    if len > 0 then
        set_colors(BLACK, CYAN)
        position(1, 1)
        puts(SCREEN, menubar & repeat(32, SCRN_COLS - len))
        start_line = iif(start_line = 1, 2, start_line)
    end if

    -- reset to default colors before return, and clear screen:
    set_colors()
    if start_line > 0 then
        scroll(SCRN_ROWS, start_line, SCRN_ROWS)
    end if
end procedure



-- PURPOSE: clear all keys, especially escape sequences, from the keys buffer.
--          i.e. make sure buffer is empty before getting new input.
procedure clear_keyboard()
    while get_key() != -1 do
    end while
end procedure



-- PURPOSE: advanced prompt_string() with backspace & colors support.
-- DETAILS: pro is prompt and its text and bk colors;
--          str is answer and its text and bk colors;
--          def is default string to return if str is empty.
--          if display_and_exit then don't wait for user's input.
-- RETURN:  ascii_string; trimmed answer without any control chars in it.
function prompt_str(
            string pro = "",       integer p_fg = WHITE,  integer p_bg = BLACK,
            ascii_string str = "", integer s_fg = YELLOW, integer s_bg = BLACK,
            ascii_string def = "",
            boolean display_and_exit = FALSE
            )

    sequence pos
    integer  chr = 0, row, col

    -- display prompt and string:
    set_colors(p_fg, p_bg)
    puts(SCREEN, pro)

    pos = get_position()
    row = pos[1]
    col = pos[2]

    set_colors(s_fg, s_bg)
    puts(SCREEN, str)

    -- edit string:
    while not display_and_exit and (chr != '\n') do
        chr = wait_key()

        if (chr = 8 or chr = 127) and length(str) then -- delete last char
            str = str[1..$ - 1]
            -- remove last char from screen:
            set_colors(s_fg, s_bg)
            position(row, col)
            puts(SCREEN, str)
            set_colors()
            puts(SCREEN, 32)

        elsif chr >= 32 and chr < 127 then
            str &= chr
        end if

        -- display string and position cursor at the end:
        set_colors(s_fg, s_bg)
        position(row, col)
        puts(SCREEN, str)
    end while

    set_colors()    -- reset back to default colors.

    str = trim(str)
    if length(str) = 0 and length(def) > 0 then
        return trim(def)
    else
        return str
    end if
end function



-- PURPOSE: calling prompt_str() with display_and_exit is TRUE.
--          this simply display everything but don't wait for any input.
-- DETAILS: arguments usage is the same as in prompt_str().
procedure display_str(
            string pro = "",       integer p_fg = WHITE,        integer p_bg = BLACK,
            ascii_string str = "", integer s_fg = BRIGHT_WHITE, integer s_bg = BLACK
            )

    if atom(prompt_str(pro, p_fg, p_bg, str, s_fg, s_bg, "", TRUE)) then
    end if
end procedure



-- GET/PUT DATA IN FILE =====================================================

-- PURPOSE: for initializing data sequence or adding a new entry to data.
constant EMPTY_DATA = {{"", ""}}



-- PURPOSE: initialize or get saved expressions from data file.
-- RETURN:  sequence; data: { {"description1", "expression1"},
--                            {"description2", "expression2"}, ... }
-- SEEALSO: save_data
constant
    -- example of expression line: "description {$@$} expression\n".
    SEPARATOR = "{$@$}",
    -- in case of error in file - don't load lines over maximum length:
    MAX_LINE_LENGTH = 1000      -- 1000 chars is a sane length.
function get_data()
    sequence data = {}
    integer  fn, pos, len_sep = length(SEPARATOR)
    object   line

    fn = open(DATA_FILE, "r")

    if fn > 2 then
        while TRUE do
            line = gets(fn)

            if atom(line) then
                close(fn)
                exit
            else
                line = trim(line)

                if length(line) > 0 and length(line) <= MAX_LINE_LENGTH then
                    pos = match(SEPARATOR, line)
                    if pos then
                        data = append(data, {
                            trim(line[1..pos - 1]),
                            trim(line[pos + len_sep..$])
                            }
                        )
                    else
                        data = append(data, {line, ""})
                    end if
                end if
            end if

        end while

    else
        display_str("\nError: cannot find or open '" & DATA_FILE & "'!\n", YELLOW)
        any_key()

        -- initialize data with two demo expressions:
        data = {
            {"Percents of total --> Part of total", "total * (percent / 100)"},
            {"Part of total --> Percents of total", "(part / total) * 100"   }
        }
    end if

    if length(data) = 0 then
        return EMPTY_DATA   -- initialize data to a single entry.
    else
        return data
    end if
end function



-- PURPOSE: save expressions in data file.
-- SEEALSO: get_data
procedure save_data(sequence data)
    integer fn

    fn = open(DATA_FILE, "w")

    if fn > 2 then
        for i = 1 to length(data) do
            puts(fn, data[i][1] & 32 & SEPARATOR & 32 & data[i][2] & '\n')
        end for

        close(fn)
    else
        display_str("\nError: cannot save '" & DATA_FILE & "'!\n", YELLOW)
        any_key()
    end if
end procedure



-- MENU ITEMS ===============================================================

-- PURPOSE: display a single expression.
procedure display_expr(sequence data, integer ix)
    position(2, 1)
    display_str("Entry #:     "  , , , sprintf("%d", ix), BRIGHT_CYAN)
    display_str("\nDescription: ", , , data[ix][1], BRIGHT_WHITE)
    display_str("\nExpression:  ", , , data[ix][2], YELLOW, BLUE)
end procedure



-- PURPOSE: add new expression, or edit existing expression.
-- DETAILS: if edit = TRUE then edit existing expression else add new expr.
-- RETURN:  sequence; data
-- SEEALSO: menu
function add_or_edit_expr(sequence data, integer ix, boolean edit)
    if edit then
        clear_console(0, " Edit expression ")
        position(6, 1)
        display_str("* Enter new data (Backspace = delete):\n", BRIGHT_CYAN)
    else
        clear_console(2, " Add new expression ")
        display_expr(data, ix)
        position(2, 1)
    end if

    -- prompt for strings:
    data[ix][1] = prompt_str("\nDescription: ", , , data[ix][1], BRIGHT_WHITE)
    data[ix][2] = prompt_str("\nExpression:  ", , , data[ix][2], YELLOW, BLUE)

    return data
end function



-- PURPOSE: remove expression.
-- RETURN:  sequence; data
-- SEEALSO: menu
function remove_expr(sequence data, integer ix)
    clear_console(0, " Remove expression")

    position(6, 1)
    if equal(upper(prompt_str("Remove expression [y, N]? ", YELLOW, RED)), "Y") then
        data = data[1..ix - 1] & data[ix + 1..$]
    end if

    if length(data) = 0 then
        return EMPTY_DATA   -- initialize data to a single entry.
    else
        return data
    end if
end function



-- PURPOSE: let Euphoria interpreter (eui) calculate expression, for comparing
--          to math_eval() result.
-- DETAILS: v is variables sequence: { {"var_name1", "var_name2", ...},
--                                     {"var_expr1", "var_expr2", ...}, ... }
-- SEEALSO: calc_expr
procedure eui_calc_expr(sequence data, integer ix, sequence v)
    integer  fn, nul

    fn = open(EU_TEMP_FILE, "w")

    -- create temporary ex file with variable-declarations and expression:
    if fn > 2 then
        puts(fn,
            "-- this is a temporary file and may be deleted safely --\n\n" &
            "include std/math.e\n\n"
        )

        -- declare and assign values or expressions to variable names:
        for i = 1 to length(v[1]) do
            puts(fn, "atom " & v[1][i] & " = " & v[2][i] & '\n')
        end for

        -- print result on screen:
        puts(fn, "\nprintf(1, \"-->  eui Result = %g\", \n\t" &
                 data[ix][2] & "\n)"
        )

        close(fn)

        -- run Euphoria interpreter to show its result:
        set_colors(BRIGHT_GREEN)
        nul = system_exec(EU_INTERPRETER & 32 & EU_TEMP_FILE, 2)

    else
        display_str("\nError: cannot save '" & EU_TEMP_FILE & "'!\n", YELLOW)
        any_key()
    end if
end procedure



-- PURPOSE: calculate expression using math_eval() function.
-- DETAILS: if deg then use degrees else use radians in calculations.
--          if eui then compare result of math_eval() with Euphoria interpreter.
-- SEEALSO: eui_calc_expr
procedure calc_expr(sequence data, integer ix, boolean deg, boolean eui)
    object   result
    sequence v, id = get_identifiers(data[ix][2], GET_ID_VARIABLES)

    clear_console(0, " Calculate expression ")
    position(5, 1)

    if id[1] = GET_ID_SUCCESS then
        -- sequence for storing user variables:
        v = {
            id[2],                      -- [1] variable names
            repeat("", length(id[2]))   -- [2] variable values or expressions
        }

        -- prompt for variables value/expression:
        for i = 1 to length(v[1]) do
            v[2][i] = prompt_str("\nValue/Expr of '" & v[1][i] &
                                                        "': ", , , , YELLOW)
        end for

        -- calculate expression:
        result = math_eval(data[ix][2], deg, v)

    else -- id[1] = GET_ID_FAIL:
        result = id[2]  -- error message string
        v = {{}, {}}    -- initialize variables {names, values} for 'eui'
    end if

    -- print result atom or error message:
    display_str("\n--> eval Result = ", BRIGHT_WHITE)
    if atom(result) then
        display_str( , , , sprintf("%g\n", result), YELLOW)
    else
        display_str( , , , result & '\n', BRIGHT_WHITE, RED)
    end if

    -- compare result with Euphoria result:
    if eui then
        eui_calc_expr(data, ix, v)
    end if

    set_colors()
    any_key("")
end procedure



-- PURPOSE: display usage help for the program.
procedure display_help()
    clear_console(2)

    position(3, 1)
    puts(SCREEN,
        "  " & MATH_EVAL_VERSION & "\n\n" &
        "* Press N or B to browse between expressions.\n" &
        "* Press C to calculate the current expression\n" &
        "  - value, expression, and previous variables may be\n" &
        "    assigned to variable.\n" &
        "* Press E to edit (modify) the current expression.\n" &
        "* Press A to add a new expression after the current one\n" &
        "  - you may include variables in the expression (e.g. 'x_2').\n" &
        "* Press R to remove (delete) the current expression.\n" &
        "* Press U to switch units between Radians and Degrees\n" &
        "  - Degrees are not available if 'euI' is 'y'; Use\n" &
        "    external calculator to compare the result.\n" &
        "* Press I to compare the result with Euphoria interpreter (eui).\n" &
        "* Press Q to save the expressions and quit the program.\n" &
        "* Press H to display this help.\n" &
        "\n  For a complete list of operators, functions and constants,\n" &
        "  please read the user manual 'math_eval.doc'. "
    )

    any_key("")
end procedure



-- MAIN =====================================================================

constant
    HOTKEYS  = "NBCEARUIQH",
    -- menu bar *maximum* length should be the standard 80 columns.
    MENU_BAR =
" Next/Back | Calc | Edit | Add | Remove | Unit:     | euI:   | Quit | Help "



-- PURPOSE: verify a reasonable size of screen before we run the program.
--          display message & abort if not (more friendly then crash message).
-- SEEALSO: clear_console
procedure verify_screen_size()
    sequence st_msg = "\nNot enough ",
             en_msg = " on screen for proper display!\n"

    if SCRN_COLS < length(MENU_BAR) then
        st_msg &= "columns"
    elsif SCRN_ROWS < 15 then  -- 15 lines is reasonable.
        st_msg &= "lines"
    else
        return
    end if

    display_str(st_msg & en_msg, YELLOW, BLACK)
    any_key()
    abort(1)
end procedure



-- PURPOSE: clear screen and display menu bar on top of screen.
-- DETAILS: if deg then use degrees else use radians in calculations.
--          if eui then compare result of math_eval() with Euphoria interpreter.
-- SEEALSO: menu
procedure display_menu_bar(boolean deg, boolean eui)
    -- clear the whole screen and display menu line:
    clear_console(2, MENU_BAR)

    -- display booleans:
    position(1, find('U', MENU_BAR) + 6)
    display_str(iif(deg, "deg", "rad"), BRIGHT_WHITE, CYAN)
    position(1, find('I', MENU_BAR) + 3)
    display_str(iif(eui, "y", "n"), BRIGHT_WHITE, CYAN)

    -- display hot keys:
    set_colors(YELLOW, CYAN)
    for i = 1 to length(HOTKEYS) do
        position(1, find(HOTKEYS[i], MENU_BAR))
        puts(SCREEN, HOTKEYS[i])
    end for

    set_colors()    -- reset colors.
end procedure



-- PURPOSE: startup routine that runs the program.
procedure menu()
    sequence            -- data stores all the expressions in memory.
        data = get_data()
    integer
        ix = 1,         -- current index in data (current expression). init to 1.
        len_data,       -- length of data sequence.
        key             -- menu hot key pressed by user
    boolean
        deg = FALSE,    -- FALSE=radians/TRUE=degrees. init to use radians.
        eui = FALSE     -- TRUE=compare result of math_eval() with Euphoria interpreter.


    -- on start, verify screen size, then directly display the help:
    verify_screen_size()
    display_menu_bar(deg, eui)
    display_help()


    -- enter the main loop (menu):
    while TRUE do
        len_data = length(data)

        -- eui (Euphoria interpreter) is using only radians:
        if eui then
            deg = FALSE
        end if

        -- always clear screen & display the current expression (data[ix]):
        display_menu_bar(deg, eui)
        display_expr(data, ix)

        -- move cursor to standby position 1,1 and wait for hot key:
        position(1, 1)
        key = -1
        while not find(key, HOTKEYS) do
            clear_keyboard()           -- clear possible escape sequences.
            key = upper(wait_key())
        end while

        -- do menu item:
        switch key do
            case 'N' then
                ix = iif(ix + 1 <= len_data, ix + 1, 1)
            case 'B' then
                ix = iif(ix - 1 >= 1, ix - 1, len_data)
            case 'C' then
                calc_expr(data, ix, deg, eui)
            case 'E' then
                data = add_or_edit_expr(data, ix, TRUE)
            case 'A' then
                data = data[1..ix] & EMPTY_DATA & data[ix + 1..$]
                ix += 1
                data = add_or_edit_expr(data, ix, FALSE)
            case 'R' then
                data = remove_expr(data, ix)
                len_data = length(data)
                ix = iif(ix > len_data, len_data, ix)
            case 'U' then
                deg = not deg
            case 'I' then
                eui = not eui
            case 'Q' then
                save_data(data)
                clear_console()
                abort(0)
            case 'H' then
                display_help()
        end switch
    end while
end procedure



menu()  -- start the program.



-- End of file.
